"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var service_1 = require("../service");
var Leaf = require("leaf-ts");
var crypto = require("crypto");
var fs = require("fs");
var pathModule = require("path");
exports.Errors = Leaf.ErrorDoc.build({
    blobError: {},
    unknowError: {},
    notFound: {}
});
var BlobService = /** @class */ (function (_super) {
    __extends(BlobService, _super);
    function BlobService(settings) {
        var _this = _super.call(this) || this;
        _this.settings = settings;
        _this.name = "BlobService";
        _this.dependencies = ["IncrementalIdService", "MongodbService", "ExpressService"];
        _this.collectionName = "blob";
        _this.route = "";
        _this.path = "";
        _this.configRouteAndPath();
        return _this;
    }
    BlobService.prototype.configRouteAndPath = function () {
        if (this.settings.blob && this.settings.blob.blobStorageDir) {
            this.path = this.settings.blob.blobStorageDir;
        }
        else {
            this.path = pathModule.join(__dirname, "../../../../../blob");
        }
        if (this.settings.blob && this.settings.blob.webAccessRoute) {
            this.route = this.settings.blob.webAccessRoute;
        }
        else {
            this.route = "/blob:id";
        }
    };
    BlobService.prototype.initialize = function (done) {
        var _this = this;
        this.blobCollection = this.services.MongodbService.db.collection(this.collectionName);
        this.services.IncrementalIdService.ensure({
            name: "blob",
            offset: 100000
        }, function () {
            fs.stat(_this.path, function (err) {
                if (err) {
                    fs.mkdirSync(_this.path);
                }
            });
            done();
        });
        this.services.ExpressService.server.use(this.route, function (req, res) {
            var id = req.params.id;
            _this.getBlob({ id: id }, function (err, blob) {
                if (err || blob == null) {
                    res.status(404).send(err);
                }
                else {
                    var path_1 = _this.path + "/" + blob.hash + "." + blob.format;
                    if (req.query.download) {
                        var stats = fs.statSync(path_1);
                        if (stats.isFile()) {
                            res.set({
                                'Content-Type': 'application/octet-stream',
                                'Content-Disposition': 'attachment; filename=' + encodeURIComponent(blob.name),
                                'Content-Length': stats.size
                            });
                            fs.createReadStream(path_1).pipe(res);
                        }
                        else {
                            res.status(404).send("download resource is not exist");
                        }
                    }
                    else {
                        fs.stat(path_1, function (err, result) {
                            if (err) {
                                res.status(404);
                                res.send(err);
                            }
                            else {
                                res.status(200).sendFile(path_1);
                            }
                        });
                    }
                }
            });
        });
    };
    BlobService.prototype.redirectToServerPath = function (req, res) {
        var id = req.params.blobId;
        //console.log("redirectId", id)
        this.getBlob(id, function (err, result) {
            if (err) {
                res.end();
            }
            else {
                res.redirect(result.url);
            }
        });
    };
    BlobService.prototype.createBlob = function (option, callback) {
        var _this = this;
        this.createBlobFileByHardLink(option.path, function (err, blob) {
            if (err) {
                callback(err);
                return;
            }
            _this.services.IncrementalIdService.next({ name: "blob" }, function (err, id) {
                if (err) {
                    callback(new exports.Errors.unknowError("Fail to create id", { via: err }));
                    return;
                }
                blob.name = option.name;
                blob.id = id.toString();
                blob.url = "/blob/" + blob.id;
                _this.blobCollection.insert(blob, function (err, result) {
                    if (err) {
                        callback(new exports.Errors.unknowError("Fail to insert blob", { via: err }));
                        return;
                    }
                    callback(null, blob);
                });
            });
        });
    };
    BlobService.prototype.createBlobFileByHardLink = function (path, callback) {
        var _this = this;
        var hash = crypto.createHash("md5");
        var stream = fs.createReadStream(path);
        var blob = {};
        var pathArray = path.split("/");
        var fileName = pathArray[pathArray.length - 1];
        var fileNameArray = fileName.split(".");
        var format = fileNameArray[fileNameArray.length - 1];
        blob.format = format;
        stream.on("data", function (data) {
            hash.update(data);
        });
        stream.on("error", function (err) {
            stream.destroy();
            callback(new exports.Errors.blobError("read uploadFile fail", { via: err }));
        });
        stream.on("end", function () {
            var hex = hash.digest("hex");
            //let hex = hash.update(Date.now() + blob.name).digest("hex")
            blob.hash = hex;
            var newPath = _this.path + "/" + hex + "." + format;
            fs.stat(newPath, function (err, stats) {
                if (err) {
                    fs.link(path, newPath, function (err) {
                        if (err) {
                            callback(new exports.Errors.blobError("create link fail", { via: err }));
                            return;
                        }
                        callback(null, blob);
                    });
                }
                else {
                    callback(null, blob);
                }
            });
        });
    };
    BlobService.prototype.getBlob = function (option, callback) {
        this.blobCollection.findOne({ id: option.id }, function (err, blob) {
            if (err || !blob) {
                callback(new exports.Errors.notFound("not fond in mongodb"));
                return;
            }
            callback(null, blob);
        });
    };
    BlobService.prototype.getBlobDownloadUrl = function (option, callback) {
        var _this = this;
        this.getBlob({ id: option.id }, function (err, result) {
            if (err) {
                callback(err);
            }
            else {
                var route = "/" + _this.route.split("/")[1] + ("/" + option.id);
                var url = route + "?download=true";
                callback(null, url);
            }
        });
    };
    BlobService.prototype.getBlobUrl = function (blob, callback) {
        var blobId = blob.id;
        this.blobCollection.findOne({ id: blobId }, function (err, blob) {
            if (err || !blob) {
                callback(new exports.Errors.notFound("not font in mongodb"));
                return;
            }
            callback(null, blob.url);
        });
    };
    return BlobService;
}(service_1.Service));
exports.BlobService = BlobService;
//# sourceMappingURL=blobService.js.map