import { APIRoute, MiddlewareRoute, RouteService, Method } from "../../route";
import { ExpressService } from "../expressService";
import { LegacyUserService } from "../legacyUserService";
import { LegacyUserDefinitionService, LegacyUser } from "../legacyUserDefinitionService";
import { RouteSessionService } from "../routeSessionService";
import * as express from "express";
export declare const Errors: any;
export declare class LegacyUserAPIService<TLegacyUser extends LegacyUser = LegacyUser> extends RouteService {
    readonly name: string;
    dependencies: string[];
    prefix: string;
    services: {
        ExpressService: ExpressService;
        RouteSessionService: RouteSessionService;
        LegacyUserService: LegacyUserService<TLegacyUser>;
        LegacyUserDefinitionService: LegacyUserDefinitionService<TLegacyUser>;
    };
    initialize(): void;
}
export declare class SessionLegacyUserProvider extends MiddlewareRoute {
    services: {
        LegacyUserService: LegacyUserService<LegacyUser>;
        RouteSessionService: RouteSessionService;
    };
    req: express.Request & {
        session: {
            legacyUsername: string;
        };
    };
    info: {
        legacyUser: LegacyUser;
    };
    handle(): void;
}
export declare class Register extends APIRoute<null> {
    readonly method: Method<"POST">;
    readonly path: string;
    readonly minLegacyUsernameLength: number;
    readonly maxLegacyUsernameLength: number;
    readonly minPasswordLength: number;
    readonly maxPasswordLength: number;
    services: {
        LegacyUserService: LegacyUserService<LegacyUser>;
        RouteSessionService: RouteSessionService;
        LegacyUserDefinitionService: LegacyUserDefinitionService<LegacyUser>;
    };
    body: Partial<LegacyUser>;
    private checkLegacyUserValidation(legacyUser);
    req: express.Request & {
        session: {
            legacyUsername: string;
        };
    };
    info: {
        legacyUser: LegacyUser;
    };
    handle(): void;
}
export declare class GetLegacyUserProfile extends APIRoute<LegacyUser> {
    readonly method: Method<"GET">;
    readonly path: string;
    services: {
        LegacyUserService: LegacyUserService<LegacyUser>;
    };
    info: {
        legacyUser: LegacyUser;
    };
    handle(): void;
}
export declare class Login extends APIRoute<LegacyUser> {
    readonly method: Method<"PUT">;
    readonly path: string;
    services: {
        LegacyUserService: LegacyUserService<LegacyUser>;
    };
    body: {
        login: string;
        password: string;
    };
    req: express.Request & {
        session: {
            legacyUsername: string;
        };
    };
    handle(): void;
}
export declare class Logout extends APIRoute<null> {
    readonly method: Method<"DELETE">;
    readonly path: string;
    req: express.Request & {
        session: {
            legacyUsername: string;
        };
    };
    services: {
        LegacyUserService: LegacyUserService<LegacyUser>;
    };
    handle(): void;
}
