import { Service } from "../service"
import * as express from "express"
import { MongodbService } from "./mongodbService"
import { ExpressService } from "./expressService"
import * as ConnectMongo from "connect-mongo"
import * as ExpressSession from "express-session"
const MongoStore = ConnectMongo(ExpressSession);
import * as Session from "express-session"

export class RouteSessionService extends Service {
    name: "RouteSessionService" = "RouteSessionService"
    dependencies: ["ExpressService", "MongodbService"] = ["ExpressService", "MongodbService"]
    services: {
        ExpressService: ExpressService
        MongodbService: MongodbService
    }
    asBuiltIn() {
        console.log(`RouteService default use MongodbService and default secret: ${this.config.secret || this.defaultSecret}`)
    }
    constructor(config: {
        http?: {
            session?: {
                secret?: string
                maxAge?: number
                resave?: boolean
                saveUninitialized?: boolean
                store?: Session.Store
            }
        }
    } = {}) {
        super()
        let http = config.http || {}
        this.config = http.session || {}
    }
    private config: {
        secret?: string
        resave?: boolean
        saveUninitialized?: boolean
        maxAge?: number
        store?: Session.Store
    } = {}
    private defaultSecret = "default-session-secret"
    // 31 day
    private defaultMaxAge = 1000 * 60 * 60 * 24 * 31
    initialize() {
        this.services.ExpressService.server.use(ExpressSession({
            secret: this.config.secret || this.defaultSecret
            , store: this.config.store || new MongoStore({ db: this.services.MongodbService.db, ttl: this.config.maxAge || this.defaultMaxAge })
            , resave: this.config.resave || false
            , saveUninitialized: this.config.saveUninitialized || false
        }))
    }
}
