"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var Leaf = require("leaf-ts");
var service_1 = require("../service");
var multer = require("multer");
var fs = require("fs");
var pathModule = require("path");
exports.Errors = Leaf.ErrorDoc.build({
    upload: {},
});
//uploadBlobService
var UploadBlobService = /** @class */ (function (_super) {
    __extends(UploadBlobService, _super);
    function UploadBlobService(settings) {
        var _this = _super.call(this) || this;
        _this.settings = settings;
        _this.dependencies = ["MongodbService", "ExpressService", "BlobService"];
        _this.route = "";
        _this.path = "";
        _this.configRouteAndPath();
        return _this;
    }
    UploadBlobService.prototype.initialize = function () {
        var _this = this;
        if (!fs.existsSync(this.path)) {
            fs.mkdirSync(this.path);
        }
        var storageConfig = this.createMulterStorage();
        var multerMiddleware = multer({ storage: storageConfig }).single("file");
        this.services.ExpressService.server.post(this.route, function (req, res) {
            multerMiddleware(req, res, function (err) {
                if (err) {
                    res.status(400);
                    res.send(new exports.Errors.upload(err));
                    return;
                }
                if (!req.file) {
                    res.status(400);
                    res.send(new exports.Errors.upload("have not find file in upload"));
                    return;
                }
                var fileName = req.file.filename;
                var originalName = req.file.originalname;
                var saveFilePath = pathModule.join(_this.path, fileName);
                fs.stat(saveFilePath, function (err, stat) {
                    if (err) {
                        res.status(500).send(err);
                    }
                    else {
                        _this.services.BlobService.createBlob({ path: saveFilePath, name: originalName }, function (err, blob) {
                            if (err) {
                                res.status(500);
                                res.send(err);
                                return;
                            }
                            _this.services.BlobService.getBlobUrl(blob, function (err, url) {
                                if (err) {
                                    res.status(500).send(err);
                                    return;
                                }
                                res.status(200).send(JSON.stringify({
                                    state: blob && true || false,
                                    error: err,
                                    data: blob,
                                    path: url
                                }));
                            });
                            // fs.unlink(saveFilePath, (err) => {
                            //     if (err) {
                            //         console.log("remove tmp file failed", err)
                            //         return
                            //     }
                            //     console.log("remove tmp file success")
                            // })
                        });
                    }
                });
            });
        });
        this.services.ExpressService.server.get(this.route, function (req, res) {
            var id = req.query.blobId;
            _this.services.BlobService.getBlobDownloadUrl({ id: id }, function (err, downloadUrl) {
                if (err) {
                    res.status(404).send(err);
                }
                else {
                    res.status(200).json({ url: downloadUrl });
                }
            });
        });
    };
    UploadBlobService.prototype.configRouteAndPath = function () {
        if (this.settings.blob && this.settings.blob.upload && this.settings.blob.upload.tmpStoragePath) {
            this.path = this.settings.blob.upload.tmpStoragePath;
        }
        else {
            this.path = "/tmp/upload";
        }
        if (this.settings.blob && this.settings.blob.upload && this.settings.blob.upload.webUploadRoute) {
            this.route = this.settings.blob.upload.webUploadRoute;
        }
        else {
            this.route = "/uploadBlob";
        }
    };
    UploadBlobService.prototype.createMulterStorage = function () {
        var _this = this;
        var storage = multer.diskStorage({
            destination: function (req, file, cb) {
                cb(null, _this.path);
            },
            filename: function (req, file, cb) {
                var fileFormat = (file.originalname).split('.');
                cb(null, "uploadBlob" + '-' + Date.now() + '.' + fileFormat[fileFormat.length - 1]);
            }
        });
        return storage;
    };
    return UploadBlobService;
}(service_1.Service));
exports.UploadBlobService = UploadBlobService;
//# sourceMappingURL=uploadBlobService.js.map