
import * as Leaf from "leaf-ts"
import { Service } from "../service"
import * as multer from "multer"
import * as fs from "fs"
import * as pathModule from "path"
import { BlobService } from "./blobService";

export const Errors = Leaf.ErrorDoc.build({
    upload: {},
})
//uploadBlobService
export class UploadBlobService extends Service {
    name: "UploadBlobService"
    dependencies = ["MongodbService", "ExpressService", "BlobService"]
    services: { ExpressService, BlobService }
    route: string = ""
    path: string = ""

    constructor(private readonly settings: {
        blob: {
            upload: {
                webUploadRoute: string,
                tmpStoragePath?: string
            }
        }
    }) {
        super()
        this.configRouteAndPath()
    }

    initialize() {
        if (!fs.existsSync(this.path)) {
            fs.mkdirSync(this.path)
        }
        let storageConfig = this.createMulterStorage()
        let multerMiddleware = multer({ storage: storageConfig }).single("file")
        this.services.ExpressService.server.post(this.route, (req, res) => {
            multerMiddleware(req, res, (err) => {
                if (err) {
                    res.status(400)
                    res.send(new Errors.upload(err))
                    return
                }
                if (!req.file) {
                    res.status(400)
                    res.send(new Errors.upload("have not find file in upload"))
                    return
                }
                let fileName = req.file.filename
                let originalName = req.file.originalname
                let saveFilePath = pathModule.join(this.path, fileName)
                fs.stat(saveFilePath, (err, stat) => {
                    if (err) {
                        res.status(500).send(err)
                    } else {
                        this.services.BlobService.createBlob({ path: saveFilePath, name: originalName }, (err, blob) => {
                            if (err) {
                                res.status(500)
                                res.send(err)
                                return
                            }
                            this.services.BlobService.getBlobUrl(blob, (err, url) => {
                                if (err) {
                                    res.status(500).send(err)
                                    return
                                }
                                res.status(200).send(JSON.stringify({
                                    state: blob && true || false,
                                    error: err,
                                    data: blob,
                                    path: url
                                }))
                            })
                            // fs.unlink(saveFilePath, (err) => {
                            //     if (err) {
                            //         console.log("remove tmp file failed", err)
                            //         return
                            //     }
                            //     console.log("remove tmp file success")
                            // })
                        })
                    }
                })
            })
        })

        this.services.ExpressService.server.get(this.route, (req, res) => {
            let id = req.query.blobId
            this.services.BlobService.getBlobDownloadUrl({ id: id }, (err, downloadUrl) => {
                if (err) {
                    res.status(404).send(err)
                } else {
                    res.status(200).json({ url: downloadUrl })
                }
            })
        })
    }

    private configRouteAndPath() {
        if (this.settings.blob && this.settings.blob.upload && this.settings.blob.upload.tmpStoragePath) {
            this.path = this.settings.blob.upload.tmpStoragePath
        } else {
            this.path = "/tmp/upload"
        }

        if (this.settings.blob && this.settings.blob.upload && this.settings.blob.upload.webUploadRoute) {
            this.route = this.settings.blob.upload.webUploadRoute
        } else {
            this.route = "/uploadBlob"
        }
    }


    private createMulterStorage() {
        let storage = multer.diskStorage({
            destination: (req, file, cb) => {
                cb(null, this.path)
            },

            filename: (req, file, cb) => {
                let fileFormat = (file.originalname).split('.');
                cb(null, "uploadBlob" + '-' + Date.now() + '.' + fileFormat[fileFormat.length - 1]);
            }
        })
        return storage
    }
}